@@ -18,19 +18,19 @@ use futures::{
 };
 use rpc::dispatch::BoxedUpdateSink;
 use serde_json::error::Category as JsonErrorCategory;
-use tor_async_utils::SinkExt as _;
+use tor_async_utils::{mpsc_channel_no_memquota, SinkExt as _};
 
 use crate::{
     cancel::{Cancel, CancelHandle},
     err::RequestParseError,
     globalid::{GlobalId, MacKey},
-    msgs::{BoxedResponse, FlexibleRequest, Request, RequestId, ResponseBody},
+    msgs::{BoxedResponse, FlexibleRequest, ReqMeta, Request, RequestId, ResponseBody},
     objmap::{GenIdx, ObjMap},
     RpcMgr,
 };
 
-use tor_rpcbase as rpc;
 use tor_rpcbase::templates::*;
+use tor_rpcbase::{self as rpc, RpcError};
 
 /// An open connection from an RPC client.
 ///
@@ -41,6 +41,18 @@ use tor_rpcbase::templates::*;
 /// Specifically, the `objects` table in `Inner` hold capabilities
 /// that the client will use to do things,
 /// including an `RpcSession`.
+///
+/// # In the Arti RPC System
+///
+/// A connection to Arti.
+///
+/// This object is available as soon as you open a connection to Arti RPC,
+/// even before you authenticate.  Its ObjectId is always `"connection"`.
+///
+/// Because this object is available before authentication,
+/// it provides only those methods that you need
+/// in order to perform authentication
+/// and receive an `RpcSession`.
 #[derive(Deftly)]
 #[derive_deftly(Object)]
 pub struct Connection {
@@ -66,13 +78,6 @@ pub struct Connection {
 
     /// A reference to the manager associated with this session.
     mgr: Weak<RpcMgr>,
-
-    /// A reference to this connection itself.
-    ///
-    /// Used when we're looking up the connection within the RPC system as an object.
-    ///
-    /// TODO RPC: Maybe there is an easier way to do this while keeping `context` object-save?
-    this_connection: Weak<Connection>,
 }
 
 /// The inner, lock-protected part of an RPC connection.
@@ -87,6 +92,13 @@ struct Inner {
     /// An object map used to look up most objects by ID, and keep track of
     /// which objects are owned by this connection.
     objects: ObjMap,
+
+    /// A reference to this connection itself.
+    ///
+    /// Used when we're looking up the connection within the RPC system as an object.
+    ///
+    /// TODO RPC: Maybe there is an easier way to do this while keeping `context` object-save?
+    this_connection: Option<Weak<Connection>>,
 }
 
 /// How many updates can be pending, per connection, before they start to block?
@@ -127,6 +139,13 @@ impl ConnectionId {
 }
 
 impl Connection {
+    /// A special object ID that indicates the connection itself.
+    ///
+    /// On a fresh connection, this is the only ObjectId that exists.
+    //
+    // TODO: We might want to move responsibility for tracking this ID and its value into ObjMap.
+    const CONNECTION_OBJ_ID: &'static str = "connection";
+
     /// Create a new connection.
     pub(crate) fn new(
         connection_id: ConnectionId,
@@ -138,12 +157,12 @@ impl Connection {
             inner: Mutex::new(Inner {
                 inflight: HashMap::new(),
                 objects: ObjMap::new(),
+                this_connection: Some(Weak::clone(this_connection)),
             }),
             dispatch_table,
             connection_id,
             global_id_mac_key,
             mgr,
-            this_connection: Weak::clone(this_connection),
         })
     }
 
@@ -187,11 +206,16 @@ impl Connection {
         &self,
         id: &rpc::ObjectId,
     ) -> Result<Arc<dyn rpc::Object>, rpc::LookupError> {
-        if id.as_ref() == "connection" {
+        if id.as_ref() == Self::CONNECTION_OBJ_ID {
             let this = self
+                .inner
+                .lock()
+                .expect("lock poisoned")
                 .this_connection
+                .as_ref()
+                .ok_or_else(|| rpc::LookupError::NoObject(id.clone()))?
                 .upgrade()
-                .ok_or(rpc::LookupError::NoObject(id.clone()))?;
+                .ok_or_else(|| rpc::LookupError::NoObject(id.clone()))?;
             Ok(this as Arc<_>)
         } else {
             let local_id = self.id_into_local_idx(id)?;
@@ -266,7 +290,10 @@ impl Connection {
         // Note that the blocking behavior here is deliberate: We want _all_ of
         // these reads to start blocking when response_sink.send is blocked.
 
-        let (tx_response, mut rx_response) = mpsc::channel::<BoxedResponse>(UPDATE_CHAN_SIZE);
+        // TODO RPC should this queue participate in memquota?
+        let (tx_response, mut rx_response) =
+            mpsc_channel_no_memquota::<BoxedResponse>(UPDATE_CHAN_SIZE);
+
         let mut finished_requests = FuturesUnordered::new();
         finished_requests.push(futures::future::pending().boxed());
 
@@ -377,7 +404,7 @@ impl Connection {
         };
 
         // Create `run_method_lowlevel` future, and make it cancellable.
-        let fut = self.run_method_lowlevel(update_sender, obj, method);
+        let fut = self.run_method_lowlevel(update_sender, obj, method, meta);
         let (handle, fut) = Cancel::new(fut);
         self.register_request(id.clone(), handle);
 
@@ -423,13 +450,23 @@ impl Connection {
     async fn run_method_lowlevel(
         self: &Arc<Self>,
         tx_updates: rpc::dispatch::BoxedUpdateSink,
-        obj: rpc::ObjectId,
+        obj_id: rpc::ObjectId,
         method: Box<dyn rpc::DeserMethod>,
+        meta: ReqMeta,
     ) -> Result<Box<dyn erased_serde::Serialize + Send + 'static>, rpc::RpcError> {
-        let obj = self.lookup_object(&obj)?;
+        let obj = self.lookup_object(&obj_id)?;
+
+        if !meta.require.is_empty() {
+            // TODO RPC: Eventually, we will need a way to tell which "features" are actually
+            // available.  But for now, we have no features, so if the require list is nonempty,
+            // we can safely reject the request.
+            return Err(MissingFeaturesError(meta.require).into());
+        }
 
         let context: Arc<dyn rpc::Context> = self.clone() as Arc<_>;
-        let invoke_future = rpc::invoke_rpc_method(context, obj, method.upcast_box(), tx_updates)?;
+
+        let invoke_future =
+            rpc::invoke_rpc_method(context, &obj_id, obj, method.upcast_box(), tx_updates)?;
 
         // Note that we drop the read lock before we await this future!
         invoke_future.await
@@ -444,6 +481,26 @@ impl Connection {
     }
 }
 
+/// An error returned when an RPC request lists some feature as required,
+/// but we don't have every such feature.
+#[derive(Clone, Debug, thiserror::Error)]
+#[error("Required features not available")]
+struct MissingFeaturesError(
+    /// A list of the features that were requested but not available.
+    Vec<String>,
+);
+
+impl From<MissingFeaturesError> for RpcError {
+    fn from(err: MissingFeaturesError) -> Self {
+        let mut e = RpcError::new(
+            err.to_string(),
+            tor_rpcbase::RpcErrorKind::FeatureNotPresent,
+        );
+        e.set_datum("rpc:unsupported_features".to_string(), err.0);
+        e
+    }
+}
+
 /// A failure that results in closing a [`Connection`].
 #[derive(Clone, Debug, thiserror::Error)]
 #[non_exhaustive]
@@ -559,20 +616,29 @@ impl rpc::Context for Connection {
     }
 
     fn release_owned(&self, id: &rpc::ObjectId) -> Result<(), rpc::LookupError> {
-        let idx = self.id_into_local_idx(id)?;
+        let removed_some = if id.as_ref() == Self::CONNECTION_OBJ_ID {
+            self.inner
+                .lock()
+                .expect("Lock poisoned")
+                .this_connection
+                .take()
+                .is_some()
+        } else {
+            let idx = self.id_into_local_idx(id)?;
 
-        if !idx.is_strong() {
-            return Err(rpc::LookupError::WrongType(id.clone()));
-        }
+            if !idx.is_strong() {
+                return Err(rpc::LookupError::WrongType(id.clone()));
+            }
 
-        let removed = self
-            .inner
-            .lock()
-            .expect("Lock poisoned")
-            .objects
-            .remove(idx);
+            self.inner
+                .lock()
+                .expect("Lock poisoned")
+                .objects
+                .remove(idx)
+                .is_some()
+        };
 
-        if removed.is_some() {
+        if removed_some {
             Ok(())
         } else {
             Err(rpc::LookupError::NoObject(id.clone()))
