@@ -2,8 +2,8 @@
 //! we've checked.
 
 use std::{
-    fs::{File, OpenOptions},
-    io::{Read, Write},
+    fs::{File, Metadata, OpenOptions},
+    io::{self, Read, Write},
     path::{Path, PathBuf},
 };
 
@@ -267,6 +267,55 @@ impl CheckedDir {
         Ok(())
     }
 
+    /// Return the [`Metadata`] of the file located at `path`.
+    ///
+    /// `path` must be a relative path, containing no `..` components.
+    /// We check the file's parent directories,
+    /// and the file's permissions.
+    /// If the file exists, it must not be a symlink.
+    ///
+    /// Returns [`Error::NotFound`] if the file does not exist.
+    ///
+    /// Return an error if `path` is absent, if its permissions are incorrect[^1],
+    /// if the permissions of any of its the parent directories are incorrect,
+    /// or if it has any components that could take us outside of this directory.
+    ///
+    /// [^1]: the permissions are incorrect if the path is readable or writable by untrusted users
+    pub fn metadata<P: AsRef<Path>>(&self, path: P) -> Result<Metadata> {
+        let path = path.as_ref();
+        self.check_path(path)?;
+        let path = self.location.join(path);
+        if let Some(parent) = path.parent() {
+            self.verifier().check(parent)?;
+        }
+
+        let meta = path
+            .symlink_metadata()
+            .map_err(|e| Error::inspecting(e, &path))?;
+
+        if meta.is_symlink() {
+            // TODO: this is inconsistent with CheckedDir::open()'s behavior, which returns a
+            // FilesystemLoop io error in this case (we can't construct such an error here, because
+            // ErrorKind::FilesystemLoop is only available on nightly)
+            let err = io::Error::new(
+                io::ErrorKind::Other,
+                format!("Path {:?} is a symlink", path),
+            );
+            return Err(Error::io(err, &path, "metadata"));
+        }
+
+        if let Some(error) = self
+            .verifier()
+            .check_one(path.as_path(), PathType::Content, &meta)
+            .into_iter()
+            .next()
+        {
+            Err(error)
+        } else {
+            Ok(meta)
+        }
+    }
+
     /// Create a [`Verifier`] with the appropriate rules for this
     /// `CheckedDir`.
     pub fn verifier(&self) -> Verifier<'_> {
@@ -392,8 +441,13 @@ mod test {
 
         let e = sd.make_directory("hello/../world").unwrap_err();
         assert!(matches!(e, Error::InvalidSubdirectory));
+        let e = sd.metadata("hello/../world").unwrap_err();
+        assert!(matches!(e, Error::InvalidSubdirectory));
+
         let e = sd.make_directory("/hello").unwrap_err();
         assert!(matches!(e, Error::InvalidSubdirectory));
+        let e = sd.metadata("/hello").unwrap_err();
+        assert!(matches!(e, Error::InvalidSubdirectory));
 
         sd.make_directory("hello/world").unwrap();
     }
@@ -519,11 +573,13 @@ mod test {
 
         // Remove a file that is there, and then make sure it is gone.
         assert!(checked.read_to_string("b/f").is_ok());
+        assert!(checked.metadata("b/f").unwrap().is_file());
         checked.remove_file("b/f").unwrap();
         assert!(matches!(
             checked.read_to_string("b/f"),
             Err(Error::NotFound(_))
         ));
+        assert!(matches!(checked.metadata("b/f"), Err(Error::NotFound(_))));
         assert!(matches!(
             checked.remove_file("b/f"),
             Err(Error::NotFound(_))
@@ -542,6 +598,40 @@ mod test {
                 checked.remove_file("x/y/z"),
                 Err(Error::BadPermission(_, _, _))
             ));
+            assert!(matches!(
+                checked.metadata("x/y/z"),
+                Err(Error::BadPermission(_, _, _))
+            ));
         }
     }
+
+    #[test]
+    #[cfg(target_family = "unix")]
+    fn metadata_symlink() {
+        use crate::testing::LinkType;
+
+        let d = Dir::new();
+        d.dir("a/b");
+        d.file("a/b/f1");
+
+        d.chmod("a/b", 0o700);
+        d.chmod("a/b/f1", 0o600);
+        d.link_rel(LinkType::File, "f1", "a/b/f1-link");
+
+        let m = Mistrust::builder()
+            .ignore_prefix(d.canonical_root())
+            .build()
+            .unwrap();
+
+        let sd = m.verifier().secure_dir(d.path("a/b")).unwrap();
+
+        assert!(sd.open("f1", OpenOptions::new().read(true)).is_ok());
+
+        // Metadata returns an error if called on a symlink
+        let e = sd.metadata("f1-link").unwrap_err();
+        assert!(
+            matches!(e, Error::Io { ref err, .. } if err.to_string().contains("is a symlink")),
+            "{e:?}"
+        );
+    }
 }
