@@ -101,7 +101,7 @@ pub(crate) use {
     },
     tor_llcrypto::pk::{curve25519, ed25519},
     tor_log_ratelim::log_ratelim,
-    tor_netdir::{HsDirParams, NetDir, NetDirProvider, Relay, Timeliness},
+    tor_netdir::{HsDirParams, NetDirProvider, Relay, Timeliness},
     tor_netdoc::doc::hsdesc::{create_desc_sign_key_cert, HsDescBuilder},
     tor_netdoc::NetdocBuilder,
     tor_persist::slug::Slug,
@@ -129,7 +129,6 @@ pub(crate) use {
     crate::ipt_set::{self, PublishIptSet},
     crate::keys::expire_publisher_keys,
     crate::keys::{IptKeyRole, IptKeySpecifier, IptKeySpecifierPattern},
-    crate::netdir::{wait_for_netdir, wait_for_netdir_to_list, NetdirProviderShutdown},
     crate::publish::Publisher,
     crate::replay::ReplayError,
     crate::replay::ReplayLog,
