@@ -2,28 +2,31 @@
 //!
 //! See the [`ArtiNativeKeystore`] docs for more details.
 
+pub(crate) mod certs;
 pub(crate) mod err;
 pub(crate) mod ssh;
-#[macro_use]
-mod rel_path;
 
 use std::io::{self, ErrorKind};
 use std::path::Path;
 use std::result::Result as StdResult;
 use std::str::FromStr;
 
-use crate::keystore::{EncodableKey, ErasedKey, KeySpecifier, Keystore};
-use crate::{arti_path, ArtiPath, ArtiPathUnavailableError, KeyPath, KeystoreId, Result};
-use err::{ArtiNativeKeystoreError, FilesystemAction};
-use rel_path::RelKeyPath;
+use crate::keystore::fs_utils::{checked_op, FilesystemAction, FilesystemError, RelKeyPath};
+use crate::keystore::{EncodableItem, ErasedKey, KeySpecifier, Keystore};
+use crate::{
+    arti_path, ArtiPath, ArtiPathUnavailableError, KeyPath, KeystoreId, Result, UnknownKeyTypeError,
+};
+use certs::UnparsedCert;
+use err::ArtiNativeKeystoreError;
 use ssh::UnparsedOpenSshKey;
 
 use fs_mistrust::{CheckedDir, Mistrust};
 use itertools::Itertools;
-use tor_key_forge::KeyType;
+use tor_error::internal;
 use walkdir::WalkDir;
 
 use tor_basic_utils::PathExt as _;
+use tor_key_forge::{CertData, KeystoreItem, KeystoreItemType};
 
 /// The Arti key store.
 ///
@@ -70,11 +73,12 @@ impl ArtiNativeKeystore {
             .verifier()
             .check_content()
             .make_secure_dir(&keystore_dir)
-            .map_err(|e| ArtiNativeKeystoreError::FsMistrust {
+            .map_err(|e| FilesystemError::FsMistrust {
                 action: FilesystemAction::Init,
                 path: keystore_dir.as_ref().into(),
                 err: e.into(),
-            })?;
+            })
+            .map_err(ArtiNativeKeystoreError::Filesystem)?;
 
         // TODO: load the keystore ID from config.
         let id = KeystoreId::from_str("arti")?;
@@ -86,9 +90,9 @@ impl ArtiNativeKeystore {
     fn rel_path(
         &self,
         key_spec: &dyn KeySpecifier,
-        key_type: &KeyType,
+        item_type: &KeystoreItemType,
     ) -> StdResult<RelKeyPath, ArtiPathUnavailableError> {
-        RelKeyPath::new(&self.keystore_dir, key_spec, key_type)
+        RelKeyPath::arti(&self.keystore_dir, key_spec, item_type)
     }
 }
 
@@ -114,19 +118,19 @@ impl Keystore for ArtiNativeKeystore {
         &self.id
     }
 
-    fn contains(&self, key_spec: &dyn KeySpecifier, key_type: &KeyType) -> Result<bool> {
-        let path = rel_path_if_supported!(self.rel_path(key_spec, key_type), Ok(false));
+    fn contains(&self, key_spec: &dyn KeySpecifier, item_type: &KeystoreItemType) -> Result<bool> {
+        let path = rel_path_if_supported!(self.rel_path(key_spec, item_type), Ok(false));
 
         let meta = match checked_op!(metadata, path) {
             Ok(meta) => meta,
             Err(fs_mistrust::Error::NotFound(_)) => return Ok(false),
             Err(e) => {
-                return Err(ArtiNativeKeystoreError::FsMistrust {
+                return Err(FilesystemError::FsMistrust {
                     action: FilesystemAction::Read,
                     path: path.rel_path_unchecked().into(),
                     err: e.into(),
-                }
-                .into())
+                })
+                .map_err(|e| ArtiNativeKeystoreError::Filesystem(e).into());
             }
         };
 
@@ -134,99 +138,155 @@ impl Keystore for ArtiNativeKeystore {
         if meta.is_file() {
             Ok(true)
         } else {
-            Err(ArtiNativeKeystoreError::NotARegularFile(path.rel_path_unchecked().into()).into())
+            Err(
+                ArtiNativeKeystoreError::Filesystem(FilesystemError::NotARegularFile(
+                    path.rel_path_unchecked().into(),
+                ))
+                .into(),
+            )
         }
     }
 
-    fn get(&self, key_spec: &dyn KeySpecifier, key_type: &KeyType) -> Result<Option<ErasedKey>> {
-        let path = rel_path_if_supported!(self.rel_path(key_spec, key_type), Ok(None));
+    fn get(
+        &self,
+        key_spec: &dyn KeySpecifier,
+        item_type: &KeystoreItemType,
+    ) -> Result<Option<ErasedKey>> {
+        let path = rel_path_if_supported!(self.rel_path(key_spec, item_type), Ok(None));
 
-        let inner = match checked_op!(read_to_string, path) {
+        let inner = match checked_op!(read, path) {
             Err(fs_mistrust::Error::NotFound(_)) => return Ok(None),
-            res => res.map_err(|err| ArtiNativeKeystoreError::FsMistrust {
-                action: FilesystemAction::Read,
-                path: path.rel_path_unchecked().into(),
-                err: err.into(),
-            })?,
+            res => res
+                .map_err(|err| FilesystemError::FsMistrust {
+                    action: FilesystemAction::Read,
+                    path: path.rel_path_unchecked().into(),
+                    err: err.into(),
+                })
+                .map_err(ArtiNativeKeystoreError::Filesystem)?,
         };
 
-        let abs_path = path.checked_path()?;
-        UnparsedOpenSshKey::new(inner, abs_path)
-            .parse_ssh_format_erased(key_type)
-            .map(Some)
+        let abs_path = path
+            .checked_path()
+            .map_err(ArtiNativeKeystoreError::Filesystem)?;
+
+        match item_type {
+            KeystoreItemType::Key(key_type) => {
+                let inner = String::from_utf8(inner).map_err(|_| {
+                    let err = io::Error::new(
+                        io::ErrorKind::InvalidData,
+                        "OpenSSH key is not valid UTF-8".to_string(),
+                    );
+
+                    ArtiNativeKeystoreError::Filesystem(FilesystemError::Io {
+                        action: FilesystemAction::Read,
+                        path: abs_path.clone(),
+                        err: err.into(),
+                    })
+                })?;
+
+                UnparsedOpenSshKey::new(inner, abs_path)
+                    .parse_ssh_format_erased(key_type)
+                    .map(Some)
+            }
+            KeystoreItemType::Cert(cert_type) => UnparsedCert::new(inner, abs_path)
+                .parse_certificate_erased(cert_type)
+                .map(Some),
+            KeystoreItemType::Unknown { arti_extension } => Err(
+                ArtiNativeKeystoreError::UnknownKeyType(UnknownKeyTypeError {
+                    arti_extension: arti_extension.clone(),
+                })
+                .into(),
+            ),
+            _ => Err(internal!("unknown item type {item_type:?}").into()),
+        }
     }
 
     fn insert(
         &self,
-        key: &dyn EncodableKey,
+        key: &dyn EncodableItem,
         key_spec: &dyn KeySpecifier,
-        key_type: &KeyType,
+        item_type: &KeystoreItemType,
     ) -> Result<()> {
         let path = self
-            .rel_path(key_spec, key_type)
+            .rel_path(key_spec, item_type)
             .map_err(|e| tor_error::internal!("{e}"))?;
         let unchecked_path = path.rel_path_unchecked();
 
         // Create the parent directories as needed
         if let Some(parent) = unchecked_path.parent() {
-            self.keystore_dir.make_directory(parent).map_err(|err| {
-                ArtiNativeKeystoreError::FsMistrust {
+            self.keystore_dir
+                .make_directory(parent)
+                .map_err(|err| FilesystemError::FsMistrust {
                     action: FilesystemAction::Write,
                     path: parent.to_path_buf(),
                     err: err.into(),
-                }
-            })?;
+                })
+                .map_err(ArtiNativeKeystoreError::Filesystem)?;
         }
 
-        let key = key.as_ssh_key_data()?;
-        // TODO (#1095): decide what information, if any, to put in the comment
-        let comment = "";
+        let item_bytes: Vec<u8> = match key.as_keystore_item()? {
+            KeystoreItem::Key(key) => {
+                // TODO (#1095): decide what information, if any, to put in the comment
+                let comment = "";
+                key.to_openssh_string(comment)?.into_bytes()
+            }
+            KeystoreItem::Cert(cert) => match cert {
+                CertData::TorEd25519Cert(cert) => cert.into(),
+                _ => return Err(internal!("unknown cert type {item_type:?}").into()),
+            },
+            _ => return Err(internal!("unknown item type {item_type:?}").into()),
+        };
 
-        let openssh_key = key.to_openssh_string(comment)?;
-
-        Ok(
-            checked_op!(write_and_replace, path, openssh_key).map_err(|err| {
-                ArtiNativeKeystoreError::FsMistrust {
-                    action: FilesystemAction::Write,
-                    path: unchecked_path.into(),
-                    err: err.into(),
-                }
-            })?,
-        )
+        Ok(checked_op!(write_and_replace, path, item_bytes)
+            .map_err(|err| FilesystemError::FsMistrust {
+                action: FilesystemAction::Write,
+                path: unchecked_path.into(),
+                err: err.into(),
+            })
+            .map_err(ArtiNativeKeystoreError::Filesystem)?)
     }
 
-    fn remove(&self, key_spec: &dyn KeySpecifier, key_type: &KeyType) -> Result<Option<()>> {
+    fn remove(
+        &self,
+        key_spec: &dyn KeySpecifier,
+        item_type: &KeystoreItemType,
+    ) -> Result<Option<()>> {
         let rel_path = self
-            .rel_path(key_spec, key_type)
+            .rel_path(key_spec, item_type)
             .map_err(|e| tor_error::internal!("{e}"))?;
 
         match checked_op!(remove_file, rel_path) {
             Ok(()) => Ok(Some(())),
             Err(fs_mistrust::Error::NotFound(_)) => Ok(None),
-            Err(e) => Err(ArtiNativeKeystoreError::FsMistrust {
-                action: FilesystemAction::Remove,
-                path: rel_path.rel_path_unchecked().into(),
-                err: e.into(),
-            }
-            .into()),
+            Err(e) => Err(ArtiNativeKeystoreError::Filesystem(
+                FilesystemError::FsMistrust {
+                    action: FilesystemAction::Remove,
+                    path: rel_path.rel_path_unchecked().into(),
+                    err: e.into(),
+                },
+            ))?,
         }
     }
 
-    fn list(&self) -> Result<Vec<(KeyPath, KeyType)>> {
+    fn list(&self) -> Result<Vec<(KeyPath, KeystoreItemType)>> {
         WalkDir::new(self.keystore_dir.as_path())
             .into_iter()
             .map(|entry| {
-                let entry = entry.map_err(|e| {
-                    let msg = e.to_string();
-                    ArtiNativeKeystoreError::Filesystem {
-                        action: FilesystemAction::Read,
-                        path: self.keystore_dir.as_path().into(),
-                        err: e
-                            .into_io_error()
-                            .unwrap_or_else(|| io::Error::new(ErrorKind::Other, msg.to_string()))
-                            .into(),
-                    }
-                })?;
+                let entry = entry
+                    .map_err(|e| {
+                        let msg = e.to_string();
+                        FilesystemError::Io {
+                            action: FilesystemAction::Read,
+                            path: self.keystore_dir.as_path().into(),
+                            err: e
+                                .into_io_error()
+                                .unwrap_or_else(|| {
+                                    io::Error::new(ErrorKind::Other, msg.to_string())
+                                })
+                                .into(),
+                        }
+                    })
+                    .map_err(ArtiNativeKeystoreError::Filesystem)?;
 
                 let path = entry.path();
 
@@ -251,13 +311,14 @@ impl Keystore for ArtiNativeKeystore {
                 if let Some(parent) = path.parent() {
                     // Check the properties of the parent directory by attempting to list its
                     // contents.
-                    self.keystore_dir.read_directory(parent).map_err(|e| {
-                        ArtiNativeKeystoreError::FsMistrust {
+                    self.keystore_dir
+                        .read_directory(parent)
+                        .map_err(|e| FilesystemError::FsMistrust {
                             action: FilesystemAction::Read,
                             path: parent.into(),
                             err: e.into(),
-                        }
-                    })?;
+                        })
+                        .map_err(ArtiNativeKeystoreError::Filesystem)?;
                 }
 
                 let malformed_err = |path: &Path, err| ArtiNativeKeystoreError::MalformedPath {
@@ -271,7 +332,7 @@ impl Keystore for ArtiNativeKeystore {
                     .to_str()
                     .ok_or_else(|| malformed_err(path, err::MalformedPathError::Utf8))?;
 
-                let key_type = KeyType::from(extension);
+                let item_type = KeystoreItemType::from(extension);
                 // Strip away the file extension
                 let path = path.with_extension("");
                 // Construct slugs in platform-independent way
@@ -281,7 +342,7 @@ impl Keystore for ArtiNativeKeystore {
                     .collect::<Vec<_>>()
                     .join(&arti_path::PATH_SEP.to_string());
                 ArtiPath::new(slugs)
-                    .map(|path| Some((path.into(), key_type)))
+                    .map(|path| Some((path.into(), item_type)))
                     .map_err(|e| {
                         malformed_err(&path, err::MalformedPathError::InvalidArtiPath(e)).into()
                     })
@@ -308,19 +369,36 @@ mod tests {
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
     use crate::test_utils::ssh_keys::*;
-    use crate::test_utils::TestSpecifier;
+    use crate::test_utils::{assert_found, TestSpecifier};
     use crate::{ArtiPath, KeyPath};
+    use std::cmp::Ordering;
     use std::fs;
     use std::path::PathBuf;
     use tempfile::{tempdir, TempDir};
+    use tor_key_forge::{CertType, EncodedEd25519Cert, KeyType};
     use tor_llcrypto::pk::ed25519;
 
     #[cfg(unix)]
     use std::os::unix::fs::PermissionsExt;
 
+    impl Ord for KeyPath {
+        fn cmp(&self, other: &Self) -> Ordering {
+            match (self, other) {
+                (KeyPath::Arti(path1), KeyPath::Arti(path2)) => path1.cmp(path2),
+                _ => unimplemented!("not supported"),
+            }
+        }
+    }
+
+    impl PartialOrd for KeyPath {
+        fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+            Some(self.cmp(other))
+        }
+    }
+
     fn key_path(key_store: &ArtiNativeKeystore, key_type: &KeyType) -> PathBuf {
         let rel_key_path = key_store
-            .rel_path(&TestSpecifier::default(), key_type)
+            .rel_path(&TestSpecifier::default(), &key_type.clone().into())
             .unwrap();
 
         rel_key_path.checked_path().unwrap()
@@ -349,19 +427,6 @@ mod tests {
         (key_store, keystore_dir)
     }
 
-    macro_rules! assert_found {
-        ($key_store:expr, $key_spec:expr, $key_type:expr, $found:expr) => {{
-            let res = $key_store.get($key_spec, $key_type).unwrap();
-            if $found {
-                assert!(res.is_some());
-                // Ensure contains() agrees with get()
-                assert!($key_store.contains($key_spec, $key_type).unwrap());
-            } else {
-                assert!(res.is_none());
-            }
-        }};
-    }
-
     macro_rules! assert_contains_arti_paths {
         ([$($arti_path:expr,)*], $list:expr) => {{
             let expected = vec![
@@ -406,7 +471,7 @@ mod tests {
 
         assert_eq!(
             key_store
-                .rel_path(&TestSpecifier::default(), &KeyType::Ed25519Keypair)
+                .rel_path(&TestSpecifier::default(), &KeyType::Ed25519Keypair.into())
                 .unwrap()
                 .rel_path_unchecked(),
             PathBuf::from("parent1/parent2/parent3/test-specifier.ed25519_private")
@@ -414,7 +479,10 @@ mod tests {
 
         assert_eq!(
             key_store
-                .rel_path(&TestSpecifier::default(), &KeyType::X25519StaticKeypair)
+                .rel_path(
+                    &TestSpecifier::default(),
+                    &KeyType::X25519StaticKeypair.into()
+                )
                 .unwrap()
                 .rel_path_unchecked(),
             PathBuf::from("parent1/parent2/parent3/test-specifier.x25519_private")
@@ -433,7 +501,7 @@ mod tests {
         // Make the permissions of the test key too permissive
         fs::set_permissions(&key_path, fs::Permissions::from_mode(0o777)).unwrap();
         assert!(key_store
-            .get(&TestSpecifier::default(), &KeyType::Ed25519Keypair)
+            .get(&TestSpecifier::default(), &KeyType::Ed25519Keypair.into())
             .is_err());
 
         // Make the permissions of the parent directory too lax
@@ -446,7 +514,7 @@ mod tests {
         assert!(key_store.list().is_err());
 
         let key_spec = TestSpecifier::default();
-        let ed_key_type = &KeyType::Ed25519Keypair;
+        let ed_key_type = &KeyType::Ed25519Keypair.into();
         assert_eq!(
             key_store
                 .remove(&key_spec, ed_key_type)
@@ -516,7 +584,7 @@ mod tests {
         };
 
         let key_spec = TestSpecifier::default();
-        let ed_key_type = &KeyType::Ed25519Keypair;
+        let ed_key_type = &KeyType::Ed25519Keypair.into();
         let path = keystore_dir.as_ref().join(
             key_store
                 .rel_path(&key_spec, ed_key_type)
@@ -555,7 +623,7 @@ mod tests {
         // Now remove the key... remove() should indicate success by returning Ok(Some(()))
         assert_eq!(
             key_store
-                .remove(&TestSpecifier::default(), &KeyType::Ed25519Keypair)
+                .remove(&TestSpecifier::default(), &KeyType::Ed25519Keypair.into())
                 .unwrap(),
             Some(())
         );
@@ -571,7 +639,7 @@ mod tests {
 
         // remove() returns Ok(None) now.
         assert!(key_store
-            .remove(&TestSpecifier::default(), &KeyType::Ed25519Keypair)
+            .remove(&TestSpecifier::default(), &KeyType::Ed25519Keypair.into())
             .unwrap()
             .is_none());
         assert!(key_store.list().unwrap().is_empty());
@@ -594,7 +662,7 @@ mod tests {
         };
 
         let key_spec = TestSpecifier::new("-i-am-a-suffix");
-        let ed_key_type = KeyType::Ed25519Keypair;
+        let ed_key_type = KeyType::Ed25519Keypair.into();
 
         assert!(key_store.insert(&*key, &key_spec, &ed_key_type).is_ok());
 
@@ -620,8 +688,36 @@ mod tests {
         fs::set_permissions(parent, fs::Permissions::from_mode(0o700)).unwrap();
 
         let err = key_store
-            .contains(&TestSpecifier::default(), &KeyType::Ed25519Keypair)
+            .contains(&TestSpecifier::default(), &KeyType::Ed25519Keypair.into())
             .unwrap_err();
         assert!(err.to_string().contains("not a regular file"), "{err}");
     }
+
+    #[test]
+    fn certs() {
+        let (key_store, _keystore_dir) = init_keystore(false);
+
+        // TODO: currently, you're allowed to build a cert out of an arbitrary byte slice
+        // (including empty ones).
+        //
+        // We should implement a proper parser for tor ed25519 certs in the near future
+        // (and reject any files that don't parse).
+        const DUMMY_CERT: &[u8] = b"not really a cert...";
+        let cert = EncodedEd25519Cert::from_bytes(DUMMY_CERT);
+        // The specifier doesn't really matter.
+        let cert_spec = TestSpecifier::default();
+        assert!(key_store
+            .insert(&cert, &cert_spec, &CertType::Ed25519TorCert.into())
+            .is_ok());
+
+        let erased_cert = key_store
+            .get(&cert_spec, &CertType::Ed25519TorCert.into())
+            .unwrap()
+            .unwrap();
+        let Ok(found_cert) = erased_cert.downcast::<EncodedEd25519Cert>() else {
+            panic!("failed to downcast cert to EncodedEd25519Cert")
+        };
+
+        assert_eq!(cert, *found_cert);
+    }
 }
