@@ -4,6 +4,7 @@ use std::time::SystemTime;
 
 use super::{IntroAuthType, IntroPointDesc};
 use crate::batching_split_before::IteratorExt as _;
+use crate::doc::hsdesc::pow::PowParamSet;
 use crate::parse::tokenize::{ItemResult, NetDocReader};
 use crate::parse::{keyword::Keyword, parser::SectionRules};
 use crate::types::misc::{UnvalidatedEdCert, B64};
@@ -40,6 +41,8 @@ pub(crate) struct HsDescInner {
     //
     // Always has >= 1 and <= NUM_INTRO_POINT_MAX entries
     pub(super) intro_points: Vec<IntroPointDesc>,
+    /// A list of offered proof-of-work parameters, at most one per type.
+    pub(super) pow_params: PowParamSet,
 }
 
 decl_keyword! {
@@ -54,6 +57,7 @@ decl_keyword! {
         "enc-key-cert" => ENC_KEY_CERT,
         "legacy-key" => LEGACY_KEY,
         "legacy-key-cert" => LEGACY_KEY_CERT,
+        "pow-params" => POW_PARAMS,
     }
 }
 
@@ -66,6 +70,7 @@ static HS_INNER_HEADER_RULES: Lazy<SectionRules<HsInnerKwd>> = Lazy::new(|| {
     rules.add(CREATE2_FORMATS.rule().required().args(1..));
     rules.add(INTRO_AUTH_REQUIRED.rule().args(1..));
     rules.add(SINGLE_ONION_SERVICE.rule());
+    rules.add(POW_PARAMS.rule().args(1..).may_repeat().obj_optional());
     rules.add(UNRECOGNIZED.rule().may_repeat().obj_optional());
 
     rules.build()
@@ -255,6 +260,9 @@ impl HsDescInner {
         // Recognize `single-onion-service` if it's there.
         let is_single_onion_service = header.get(SINGLE_ONION_SERVICE).is_some();
 
+        // Recognize `pow-params`, parsing each line and rejecting duplicate types
+        let pow_params = PowParamSet::from_items(header.slice(POW_PARAMS))?;
+
         let mut signatures = Vec::new();
         let mut expirations = Vec::new();
         let mut cert_signing_key: Option<Ed25519Identity> = None;
@@ -417,6 +425,7 @@ impl HsDescInner {
         let inner = HsDescInner {
             intro_auth_types: auth_types,
             single_onion_service: is_single_onion_service,
+            pow_params,
             intro_points,
         };
         let sig_gated = SignatureGated::new(inner, signatures);
@@ -445,7 +454,7 @@ mod test {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
-    use std::iter;
+    use std::{iter, time::Duration};
 
     use hex_literal::hex;
     use itertools::chain;
@@ -455,14 +464,17 @@ mod test {
     use crate::doc::hsdesc::{
         middle::HsDescMiddle,
         outer::HsDescOuter,
+        pow::PowParams,
         test_data::{TEST_DATA, TEST_SUBCREDENTIAL},
     };
 
-    // This is the inner document from hsdesc1.txt aka TEST_DATA
-    const TEST_DATA_INNER: &str = include_str!("../../../testdata/hsdesc-inner.txt");
-
+    /// Test one particular canned 'inner' document, checking
+    /// edge cases for zero intro points and too many intro points
     #[test]
     fn inner_text() {
+        // This is the inner document from hsdesc1.txt aka TEST_DATA
+        const TEST_DATA_INNER: &str = include_str!("../../../testdata/hsdesc-inner.txt");
+
         use crate::NetdocErrorKind as NEK;
         let _desc = HsDescInner::parse(TEST_DATA_INNER).unwrap();
 
@@ -495,6 +507,145 @@ mod test {
         }
     }
 
+    /// Test parseability of an inner document generated by C tor with PoW v1
+    #[test]
+    #[cfg(feature = "hs-pow-full")]
+    fn inner_c_pow_v1() {
+        const TEST_DATA_INNER: &str = include_str!("../../../testdata/hsdesc-inner-pow-v1.txt");
+        let desc = HsDescInner::parse(TEST_DATA_INNER).unwrap();
+        let pow_params = desc
+            .1
+            .dangerously_into_parts()
+            .0
+            .dangerously_assume_wellsigned()
+            .pow_params;
+        assert_eq!(pow_params.slice().len(), 1);
+        match &pow_params.slice()[0] {
+            PowParams::V1(v1) => {
+                let expected_effort: tor_hscrypto::pow::v1::Effort = 614.into();
+                let expected_seed: tor_hscrypto::pow::v1::Seed =
+                    hex!("144e901df0841833a6e8592190849b4412f307d1565f2f137b2a5bc21a31092a").into();
+                let expected_expiry = Some(SystemTime::UNIX_EPOCH + Duration::new(1712812537, 0));
+                assert_eq!(v1.suggested_effort(), expected_effort);
+                assert_eq!(
+                    v1.seed().to_owned().dangerously_assume_timely(),
+                    expected_seed
+                );
+                assert_eq!(v1.seed().bounds().1, expected_expiry);
+            }
+            #[allow(unreachable_patterns)]
+            _ => unreachable!(),
+        }
+    }
+
+    /// Ensure the same valid v1 pow document parses with the addition of unknown schemes
+    #[test]
+    fn inner_c_pow_v1_with_unknown() {
+        const TEMPLATE: &str = include_str!("../../../testdata/hsdesc-inner-pow-v1.txt");
+        let parts = TEMPLATE.rsplit_once("\npow-params").unwrap();
+        let test_data_inner = format!("{}\npow-params x-example\npow-params{}", parts.0, parts.1);
+        let desc = HsDescInner::parse(&test_data_inner).unwrap();
+        let pow_params = desc
+            .1
+            .dangerously_into_parts()
+            .0
+            .dangerously_assume_wellsigned()
+            .pow_params;
+        assert_eq!(pow_params.slice().len(), 1);
+    }
+
+    /// Incorrect reduced document with a pow-params line that has no scheme parameter
+    #[test]
+    fn inner_pow_empty() {
+        const TEST_DATA_INNER: &str = include_str!("../../../testdata/hsdesc-inner-pow-empty.txt");
+        let err = HsDescInner::parse(TEST_DATA_INNER).map(|_| ()).unwrap_err();
+        assert_eq!(err.kind, crate::NetdocErrorKind::TooFewArguments);
+    }
+
+    /// Incorrect document with duplicated pow-params lines of the same known type
+    #[test]
+    fn inner_pow_duplicate() {
+        // Modify the canned v1 pow example from c tor, by duplicating the entire pow-params line
+        const TEMPLATE: &str = include_str!("../../../testdata/hsdesc-inner-pow-v1.txt");
+        let first_split = TEMPLATE.rsplit_once("\npow-params").unwrap();
+        let second_split = first_split.1.split_once("\n").unwrap();
+        let test_data_inner = format!(
+            "{}\npow-params{}\npow-params{}\n{}",
+            first_split.0, second_split.0, second_split.0, second_split.1
+        );
+        let err = HsDescInner::parse(&test_data_inner)
+            .map(|_| ())
+            .unwrap_err();
+        assert_eq!(err.kind, crate::NetdocErrorKind::DuplicateToken);
+    }
+
+    /// Incorrect document with an unexpected object encoded after the pow v1 scheme's pow-params
+    #[test]
+    #[cfg(feature = "hs-pow-full")]
+    fn inner_pow_v1_object() {
+        // Modify the canned v1 pow example
+        const TEMPLATE: &str = include_str!("../../../testdata/hsdesc-inner-pow-v1.txt");
+        let first_split = TEMPLATE.rsplit_once("\npow-params").unwrap();
+        let second_split = first_split.1.split_once("\n").unwrap();
+        let test_data_inner = format!(
+            "{}\npow-params{}\n-----BEGIN THING-----\n-----END THING-----\n{}",
+            first_split.0, second_split.0, second_split.1
+        );
+        let err = HsDescInner::parse(&test_data_inner)
+            .map(|_| ())
+            .unwrap_err();
+        assert_eq!(err.kind, crate::NetdocErrorKind::UnexpectedObject);
+    }
+
+    /// Document including an unrecognized pow-params line, ignored without error and not
+    /// represented in the output at all.
+    ///
+    /// Also tests that unrecognized schemes are not subject to a restriction against
+    /// duplicate appearances. (The spec allows that implementations do not need to
+    /// implement this prohibition for arbitrary scheme strings)
+    ///
+    /// TODO: We may want PowParamSet to provide a representation for arbitrary unknown PoW
+    ///       schemes, to the extent that this information may be useful for error reporting
+    ///       purposes after an onion service rendezvous fails.
+    #[test]
+    fn inner_pow_unrecognized() {
+        // Use the reduced document from inner_pow_empty() as a template
+        const TEMPLATE: &str = include_str!("../../../testdata/hsdesc-inner-pow-empty.txt");
+        let parts = TEMPLATE.rsplit_once("\npow-params").unwrap();
+        let test_data_inner = format!(
+            "{}\npow-params x-example\npow-params x-example{}",
+            parts.0, parts.1
+        );
+        let desc = HsDescInner::parse(&test_data_inner).unwrap();
+        let pow_params = desc
+            .1
+            .dangerously_into_parts()
+            .0
+            .dangerously_assume_wellsigned()
+            .pow_params;
+        assert_eq!(pow_params.slice().len(), 0);
+    }
+
+    /// Document with an unrecognized pow-params line including an object
+    #[test]
+    fn inner_pow_unrecognized_object() {
+        // Use the reduced document from inner_pow_empty() as a template
+        const TEMPLATE: &str = include_str!("../../../testdata/hsdesc-inner-pow-empty.txt");
+        let parts = TEMPLATE.rsplit_once("\npow-params").unwrap();
+        let test_data_inner = format!(
+            "{}\npow-params x-something-else with args\n-----BEGIN THING-----\n-----END THING-----{}",
+            parts.0, parts.1
+        );
+        let desc = HsDescInner::parse(&test_data_inner).unwrap();
+        let pow_params = desc
+            .1
+            .dangerously_into_parts()
+            .0
+            .dangerously_assume_wellsigned()
+            .pow_params;
+        assert_eq!(pow_params.slice().len(), 0);
+    }
+
     #[test]
     fn parse_good() -> Result<()> {
         let desc = HsDescOuter::parse(TEST_DATA)?
