@@ -7,7 +7,7 @@
 
 /// Types used for networking (async_std implementation)
 mod net {
-    use crate::traits;
+    use crate::{impls, traits};
 
     use async_std_crate::net::{TcpListener, TcpStream, UdpSocket as StdUdpSocket};
     #[cfg(unix)]
@@ -172,6 +172,23 @@ mod net {
             self.socket.local_addr()
         }
     }
+
+    impl traits::StreamOps for TcpStream {
+        fn set_tcp_notsent_lowat(&self, notsent_lowat: u32) -> IoResult<()> {
+            impls::streamops::set_tcp_notsent_lowat(self, notsent_lowat)
+        }
+    }
+
+    #[cfg(unix)]
+    impl traits::StreamOps for UnixStream {
+        fn set_tcp_notsent_lowat(&self, _notsent_lowat: u32) -> IoResult<()> {
+            Err(traits::UnsupportedStreamOp::new(
+                "set_tcp_notsent_lowat",
+                "unsupported on Unix streams",
+            )
+            .into())
+        }
+    }
 }
 
 // ==============================
