@@ -5,8 +5,11 @@
 // exists for writing unit tests.  Let's resist the temptation to add
 // things we don't need.
 
+#![forbid(unsafe_code)] // if you remove this, enable (or write) miri tests (git grep miri)
+
 use super::io::{stream_pair, LocalStream};
 use super::MockNetRuntime;
+use crate::util::mpsc_channel;
 use core::fmt;
 use tor_rtcompat::tls::TlsConnector;
 use tor_rtcompat::{CertifiedConn, NetStreamListener, NetStreamProvider, Runtime, TlsProvider};
@@ -244,7 +247,7 @@ impl MockNetwork {
             return Err(err(ErrorKind::AddrInUse));
         }
 
-        let (send, recv) = mpsc::channel(16);
+        let (send, recv) = mpsc_channel(16);
 
         let entry = ListenerEntry { send, tls_cert };
 
@@ -566,7 +569,7 @@ fn err(k: ErrorKind) -> IoError {
     IoError::new(k, MockNetError::BadOp)
 }
 
-#[cfg(test)]
+#[cfg(all(test, not(miri)))] // miri cannot simulate the networking
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
     #![allow(clippy::bool_assert_comparison)]
