@@ -1,6 +1,7 @@
 //! Internal utilities for `tor_rtmock`
 
 use derive_deftly::define_derive_deftly;
+use futures::channel::mpsc;
 
 define_derive_deftly! {
 /// Implements `Runtime` for a struct made of multiple sub-providers
@@ -51,15 +52,15 @@ define_derive_deftly! {
     }
 
     #[async_trait]
-    impl <$tgens> NetStreamProvider<tor_rtcompat::unix::SocketAddr> for $ttype {
+    impl <$tgens> NetStreamProvider<tor_general_addr::unix::SocketAddr> for $ttype {
         type Stream = FakeStream;
-        type Listener = FakeListener<tor_rtcompat::unix::SocketAddr>;
+        type Listener = FakeListener<tor_general_addr::unix::SocketAddr>;
 
-        async fn connect(&self, _addr: &tor_rtcompat::unix::SocketAddr) -> IoResult<Self::Stream> {
-            Err(tor_rtcompat::unix::NoUnixAddressSupport::default().into())
+        async fn connect(&self, _addr: &tor_general_addr::unix::SocketAddr) -> IoResult<Self::Stream> {
+            Err(tor_general_addr::unix::NoUnixAddressSupport::default().into())
         }
-        async fn listen(&self, _addr: &tor_rtcompat::unix::SocketAddr) -> IoResult<Self::Listener> {
-            Err(tor_rtcompat::unix::NoUnixAddressSupport::default().into())
+        async fn listen(&self, _addr: &tor_general_addr::unix::SocketAddr) -> IoResult<Self::Listener> {
+            Err(tor_general_addr::unix::NoUnixAddressSupport::default().into())
         }
     }
 
@@ -154,3 +155,14 @@ pub(crate) mod impl_runtime_prelude {
         NetStreamProvider, Runtime, SleepProvider, TlsProvider, UdpProvider,
     };
 }
+
+/// Wrapper for `futures::channel::mpsc::channel` that embodies the `#[allow]`
+///
+/// We don't care about mq tracking in this test crate.
+///
+/// Exactly like `tor_async_utils::mpsc_channel_no_memquota`,
+/// but we can't use that here for crate hierarchy reasons.
+#[allow(clippy::disallowed_methods)]
+pub(crate) fn mpsc_channel<T>(buffer: usize) -> (mpsc::Sender<T>, mpsc::Receiver<T>) {
+    mpsc::channel(buffer)
+}
